-- AYCT_RequirementSets
-- Author: Castar
-- DateCreated: 3/4/2019 5:30:28 PM
--------------------------------------------------------------

-------------------------
-- CUSTOM REQUIREMENTS --
-------------------------

INSERT INTO RequirementSets
(	RequirementSetId,								RequirementSetType			) VALUES
(	'PLOT_HAS_NO_VISIBLE_RESOURCES_REQUIREMENTS',	'REQUIREMENTSET_TEST_ANY'	),
(	'PLOT_HAS_NO_FEATURE_REQUIREMENTS',				'REQUIREMENTSET_TEST_ALL'	);

INSERT INTO Requirements
(	RequirementId,								RequirementType,							Likeliness,		Impact,	Inverse,	Reverse,	Persistent,		ProgressWeight,		Triggered	) VALUES
(	'REQUIRES_PLOT_HAS_ANY_IMPROVEMENT',		'REQUIREMENT_PLOT_HAS_ANY_IMPROVEMENT',		0,				0,		0,			0,			0,				1,					0			),
(	'REQUIRES_PLOT_HAS_NO_RESOURCE',			'REQUIREMENT_PLOT_HAS_ANY_RESOURCE',		0,				0,		1,			0,			0,				1,					0			),
(	'REQUIRES_PLOT_HAS_NO_VISIBLE_RESOURCE',	'REQUIREMENT_PLOT_RESOURCE_VISIBLE',		0,				0,		1,			0,			0,				1,					0			),
(	'REQUIRES_PLOT_HAS_NO_FEATURE',				'REQUIREMENT_PLOT_HAS_ANY_FEATURE',			0,				0,		1,			0,			0,				1,					0			);

INSERT INTO RequirementSetRequirements
(	RequirementSetId,									RequirementId								) VALUES
(	'PLOT_HAS_NO_VISIBLE_RESOURCES_REQUIREMENTS',		'REQUIRES_PLOT_HAS_NO_RESOURCE'				),
(	'PLOT_HAS_NO_VISIBLE_RESOURCES_REQUIREMENTS',		'REQUIRES_PLOT_HAS_NO_VISIBLE_RESOURCE'		),
(	'PLOT_HAS_NO_FEATURE_REQUIREMENTS',					'REQUIRES_PLOT_HAS_NO_FEATURE'				);


-----------------------------------------------
-- GENERATE REQUIREMENTSETS FOR ALL FEATURES --
-----------------------------------------------

 CREATE TEMPORARY TABLE AYCT_Features_Strings AS
 SELECT FeatureType AS FeatureType, substr(FeatureType, 9) AS Feature
 FROM Features
 WHERE NaturalWonder = 0 AND FeatureType <> 'FEATURE_VOLCANO';

 -- PLOT_HAS_<FEATURE>_REQUIREMENTS, PLOT_HAS_<FEATURE>_AND_IMPROVEMENT_REQUIREMENTS, PLOT_HAS_<FEATURE>_NO_IMPROVEMENT_REQUIREMENTS

-- The first has REQUIRES_PLOT_HAS_<FEATURE>, while the second and third have REQUIRES_PLOT_HAS_ANY_IMPROVEMENT 
-- and REQUIRES_PLOT_HAS_NO_IMPROVEMENT in addition to that, respectively.
INSERT INTO RequirementSets
		(RequirementSetId,										RequirementSetType			)
SELECT	'PLOT_HAS_'||Feature||'_REQUIREMENTS',					'REQUIREMENTSET_TEST_ALL'
FROM	AYCT_Features_Strings
WHERE	NOT EXISTS (SELECT 1 FROM RequirementSets WHERE RequirementSetId = 'PLOT_HAS_'||Feature||'_REQUIREMENTS');

INSERT INTO RequirementSets
		(RequirementSetId,											RequirementSetType			)
SELECT	'PLOT_HAS_'||Feature||'_NO_IMPROVEMENT_REQUIREMENTS',		'REQUIREMENTSET_TEST_ALL'
FROM	AYCT_Features_Strings
WHERE	NOT EXISTS (SELECT 1 FROM RequirementSets WHERE RequirementSetId = 'PLOT_HAS_'||Feature||'_NO_IMPROVEMENT_REQUIREMENTS');

INSERT INTO RequirementSets
		(RequirementSetId,											RequirementSetType			)
SELECT 'PLOT_HAS_'||Feature||'_ANY_IMPROVEMENT_REQUIREMENTS',		'REQUIREMENTSET_TEST_ALL'
FROM	AYCT_Features_Strings
WHERE	NOT EXISTS (SELECT 1 FROM RequirementSets WHERE RequirementSetId = 'PLOT_HAS_'||Feature||'_AND_IMPROVEMENT_REQUIREMENTS');

INSERT INTO Requirements
		(RequirementId,						RequirementType,							Likeliness,		Impact,	Inverse,	Reverse,	Persistent,		ProgressWeight,		Triggered	)
SELECT	'REQUIRES_PLOT_HAS_'||Feature,		'REQUIREMENT_PLOT_FEATURE_TYPE_MATCHES',	0,				0,		0,			0,			0,				1,					0
FROM	AYCT_Features_Strings
WHERE	NOT EXISTS (SELECT 1 FROM Requirements WHERE RequirementId = 'REQUIRES_PLOT_HAS_'||Feature);

INSERT INTO RequirementArguments
		(RequirementId,						Name,			Type,				Value			)
SELECT	'REQUIRES_PLOT_HAS_'||Feature,		'FeatureType',	'ARGTYPE_IDENTITY',	FeatureType
FROM	AYCT_Features_Strings
WHERE	NOT EXISTS (SELECT 1 FROM RequirementArguments WHERE RequirementId = 'REQUIRES_PLOT_HAS_'||Feature);

INSERT OR REPLACE INTO RequirementSetRequirements
		(RequirementSetId,							RequirementId						)
SELECT	'PLOT_HAS_'||Feature||'_REQUIREMENTS',		'REQUIRES_PLOT_HAS_'||Feature
FROM	AYCT_Features_Strings;

INSERT OR REPLACE INTO RequirementSetRequirements
		(RequirementSetId,											RequirementId						)
SELECT	'PLOT_HAS_'||Feature||'_NO_IMPROVEMENT_REQUIREMENTS',		'REQUIRES_PLOT_HAS_'||Feature
FROM	AYCT_Features_Strings
WHERE	Feature NOT IN ('JUNGLE', 'FOREST');

INSERT OR REPLACE INTO RequirementSetRequirements
		(RequirementSetId,											RequirementId						)
SELECT	'PLOT_HAS_'||Feature||'_NO_IMPROVEMENT_REQUIREMENTS',		'REQUIRES_PLOT_HAS_NO_IMPROVEMENT'
FROM	AYCT_Features_Strings
WHERE	Feature NOT IN ('JUNGLE', 'FOREST');

INSERT OR REPLACE INTO RequirementSetRequirements
		(RequirementSetId,											RequirementId						)
SELECT	'PLOT_HAS_'||Feature||'_ANY_IMPROVEMENT_REQUIREMENTS',		'REQUIRES_PLOT_HAS_'||Feature
FROM	AYCT_Features_Strings;

INSERT OR REPLACE INTO RequirementSetRequirements
		(RequirementSetId,											RequirementId						)
SELECT	'PLOT_HAS_'||Feature||'_ANY_IMPROVEMENT_REQUIREMENTS',		'REQUIRES_PLOT_HAS_ANY_IMPROVEMENT'
FROM	AYCT_Features_Strings;

-- Cleanup
DROP TABLE AYCT_Features_Strings;

------------------------------------------------
-- GENERATE REQUIREMENTSETS FOR ALL RESOURCES --
------------------------------------------------


CREATE TEMPORARY TABLE AYCT_Resources_Strings AS
SELECT ResourceType, substr(ResourceType, 10) AS ResourceName
FROM Resources;

-- PLOT_HAS_VISIBLE_<RESOURCE>_REQUIREMENTS, PLOT_HAS_IMPROVED_<RESOURCE>_REQUIREMENTS, PLOT_HAS_UNIMPROVED_<RESOURCE>_REQUIREMENTS

-- The first has REQUIRES_PLOT_HAS_<RESOURCE> and REQUIRES_PLOT_HAS_VISIBLE_RESOURCE, while the second and third
-- have REQUIRES_PLOT_HAS_ANY_IMPROVEMENT and REQUIRES_PLOT_HAS_NO_IMPROVEMENT in addition to that, respectively.
INSERT INTO RequirementSets
		(RequirementSetId,										RequirementSetType			)
SELECT	'PLOT_HAS_VISIBLE_'||ResourceName||'_REQUIREMENTS',		'REQUIREMENTSET_TEST_ALL'
FROM	AYCT_Resources_Strings;

INSERT INTO RequirementSets
		(RequirementSetId,										RequirementSetType			)
SELECT 'PLOT_HAS_IMPROVED_'||ResourceName||'_REQUIREMENTS',		'REQUIREMENTSET_TEST_ALL'
FROM AYCT_Resources_Strings;

INSERT INTO RequirementSets
		(RequirementSetId,										RequirementSetType			)
SELECT 'PLOT_HAS_UNIMPROVED_'||ResourceName||'_REQUIREMENTS',	'REQUIREMENTSET_TEST_ALL'
FROM AYCT_Resources_Strings;

INSERT INTO Requirements
		(RequirementId,							RequirementType,							Likeliness,		Impact,	Inverse,	Reverse,	Persistent,		ProgressWeight,		Triggered	)
SELECT	'REQUIRES_PLOT_HAS_'||ResourceName,		'REQUIREMENT_PLOT_RESOURCE_TYPE_MATCHES',	0,				0,		0,			0,			0,				1,					0
FROM AYCT_Resources_Strings;

INSERT INTO RequirementArguments
		(RequirementId,							Name,			Type,				Value			)
SELECT	'REQUIRES_PLOT_HAS_'||ResourceName,		'ResourceType',	'ARGTYPE_IDENTITY',	ResourceType
FROM AYCT_Resources_Strings;

INSERT INTO RequirementSetRequirements
		(RequirementSetId,										RequirementId						)
SELECT	'PLOT_HAS_VISIBLE_'||ResourceName||'_REQUIREMENTS',		'REQUIRES_PLOT_HAS_'||ResourceName
FROM AYCT_Resources_Strings;

INSERT INTO RequirementSetRequirements
		(RequirementSetId,										RequirementId						)
SELECT	'PLOT_HAS_VISIBLE_'||ResourceName||'_REQUIREMENTS',		'REQUIRES_PLOT_HAS_VISIBLE_RESOURCE'
FROM AYCT_Resources_Strings;

INSERT INTO RequirementSetRequirements
		(RequirementSetId,										RequirementId						)
SELECT	'PLOT_HAS_IMPROVED_'||ResourceName||'_REQUIREMENTS',	'REQUIRES_PLOT_HAS_'||ResourceName
FROM AYCT_Resources_Strings;

INSERT INTO RequirementSetRequirements
		(RequirementSetId,										RequirementId						)
SELECT	'PLOT_HAS_IMPROVED_'||ResourceName||'_REQUIREMENTS',	'REQUIRES_PLOT_HAS_VISIBLE_RESOURCE'
FROM AYCT_Resources_Strings;

INSERT INTO RequirementSetRequirements
		(RequirementSetId,										RequirementId						)
SELECT	'PLOT_HAS_IMPROVED_'||ResourceName||'_REQUIREMENTS',	'REQUIRES_PLOT_HAS_ANY_IMPROVEMENT'
FROM AYCT_Resources_Strings;

INSERT INTO RequirementSetRequirements
		(RequirementSetId,										RequirementId						)
SELECT	'PLOT_HAS_UNIMPROVED_'||ResourceName||'_REQUIREMENTS',	'REQUIRES_PLOT_HAS_'||ResourceName
FROM AYCT_Resources_Strings;

INSERT INTO RequirementSetRequirements
		(RequirementSetId,										RequirementId						)
SELECT	'PLOT_HAS_UNIMPROVED_'||ResourceName||'_REQUIREMENTS',	'REQUIRES_PLOT_HAS_VISIBLE_RESOURCE'
FROM AYCT_Resources_Strings;

INSERT INTO RequirementSetRequirements
		(RequirementSetId,										RequirementId						)
SELECT	'PLOT_HAS_UNIMPROVED_'||ResourceName||'_REQUIREMENTS',	'REQUIRES_PLOT_HAS_NO_IMPROVEMENT'
FROM AYCT_Resources_Strings;


-- Rice and Wheat requirements were already kinda sorta there, so we adjust them to comform
UPDATE RequirementSetRequirements SET RequirementId = 'REQUIRES_WHEAT_IN_PLOT' 
WHERE RequirementSetId IN ('PLOT_HAS_VISIBLE_WHEAT_REQUIREMENTS', 'PLOT_HAS_IMPROVED_WHEAT_REQUIREMENTS', 'PLOT_HAS_UNIMPROVED_WHEAT_REQUIREMENTS') 
AND RequirementId = 'REQUIRES_PLOT_HAS_WHEAT';

UPDATE RequirementSetRequirements SET RequirementId = 'REQUIRES_RICE_IN_PLOT' 
WHERE RequirementSetId IN ('PLOT_HAS_VISIBLE_RICE_REQUIREMENTS', 'PLOT_HAS_IMPROVED_RICE_REQUIREMENTS', 'PLOT_HAS_UNIMPROVED_RICE_REQUIREMENTS') 
AND RequirementId = 'REQUIRES_PLOT_HAS_RICE';

-- Cleanup
DROP TABLE AYCT_Resources_Strings;