-- AYCT_Tables
-- Author: Castar
-- DateCreated: 3/3/2019 9:50:16 PM
--------------------------------------------------------------


CREATE TABLE IF NOT EXISTS Improvement_ResourceYieldChanges (
	ImprovementType TEXT NOT NULL,
	ResourceType TEXT,
	YieldType TEXT NOT NULL,
	YieldChange INTEGER NOT NULL DEFAULT 0,
	PRIMARY KEY (ImprovementType, ResourceType, YieldType),
	FOREIGN KEY (ImprovementType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (ResourceType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (YieldType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS Improvement_FeatureYieldChanges (
	ImprovementType TEXT NOT NULL,
	FeatureType TEXT,
	YieldType TEXT NOT NULL,
	YieldChange INTEGER NOT NULL DEFAULT 0,
	PRIMARY KEY (ImprovementType, FeatureType, YieldType),
	FOREIGN KEY (ImprovementType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (FeatureType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (YieldType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS Technology_ResourceYieldChanges (
	TechnologyType TEXT NOT NULL,
	ResourceType TEXT NOT NULL,
	Improved BOOLEAN NOT NULL CHECK (Improved IN (0,1)) DEFAULT 0,
	Unimproved BOOLEAN NOT NULL CHECK (Unimproved IN (0,1)) DEFAULT 1,
	YieldType TEXT NOT NULL,
	YieldChange INTEGER NOT NULL DEFAULT 0,
	PRIMARY KEY (TechnologyType, ResourceType, Improved, Unimproved, YieldType),
	FOREIGN KEY (TechnologyType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (ResourceType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (YieldType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	CHECK(Improved = 1 OR Unimproved = 1)
);

CREATE TABLE IF NOT EXISTS Technology_FeatureYieldChanges (
	TechnologyType TEXT NOT NULL,
	FeatureType TEXT NOT NULL,
	Improved BOOLEAN NOT NULL CHECK (Improved IN (0,1)) DEFAULT 0,
	Unimproved BOOLEAN NOT NULL CHECK (Unimproved IN (0,1)) DEFAULT 1,
	YieldType TEXT NOT NULL,
	YieldChange INTEGER NOT NULL DEFAULT 0,
	PRIMARY KEY (TechnologyType, FeatureType, Improved, Unimproved, YieldType),
	FOREIGN KEY (TechnologyType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (FeatureType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (YieldType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	CHECK(Improved = 1 OR Unimproved = 1)
);

CREATE TABLE IF NOT EXISTS Building_ResourceYieldChanges (
	BuildingType TEXT NOT NULL,
	ResourceType TEXT NOT NULL,
	Improved BOOLEAN NOT NULL CHECK (Improved IN (0,1)) DEFAULT 0,
	Unimproved BOOLEAN NOT NULL CHECK (Unimproved IN (0,1)) DEFAULT 1,
	YieldType TEXT NOT NULL,
	YieldChange INTEGER NOT NULL DEFAULT 0,
	PRIMARY KEY (BuildingType, ResourceType, Improved, Unimproved, YieldType),
	FOREIGN KEY (BuildingType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (ResourceType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (YieldType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	CHECK(Improved = 1 OR Unimproved = 1)
);

CREATE TABLE IF NOT EXISTS Building_FeatureYieldChanges (
	BuildingType TEXT NOT NULL,
	FeatureType TEXT NOT NULL,
	Improved BOOLEAN NOT NULL CHECK (Improved IN (0,1)) DEFAULT 0,
	Unimproved BOOLEAN NOT NULL CHECK (Unimproved IN (0,1)) DEFAULT 1,
	YieldType TEXT NOT NULL,
	YieldChange INTEGER NOT NULL DEFAULT 0,
	PRIMARY KEY (BuildingType, FeatureType, Improved, Unimproved, YieldType),
	FOREIGN KEY (BuildingType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (FeatureType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (YieldType) REFERENCES Types(Type) ON DELETE CASCADE ON UPDATE CASCADE,
	CHECK(Improved = 1 OR Unimproved = 1)
);

